@@ -63,7 +63,7 @@ sub get_next_message {
     my $_msg_frame;
 
     if ( $_msg_frame = $self->{'_parser'}->get_next_frame() ) {
-        if ($_msg_frame->is_control_frame()) {
+        if ($_msg_frame->is_control()) {
             $self->_handle_control_frame($_msg_frame);
         }
         else {
@@ -85,7 +85,7 @@ sub get_next_message {
             }
 
             if ($_msg_frame->get_fin()) {
-                return Net::WebSocket::Message::create_from_frames(
+                return Net::WebSocket::Message->new(
                     splice( @{ $self->{'_fragments'} } ),
                     $_msg_frame,
                 );
@@ -94,11 +94,30 @@ sub get_next_message {
                 push @{ $self->{'_fragments'} }, $_msg_frame;
             }
         }
+
+        $_msg_frame = undef;
     }
 
     return defined($_msg_frame) ? q<> : undef;
 }
 
+sub create_message {
+    my ($self, $frame_type, $payload) = @_;
+
+    require Net::WebSocket::FrameTypeName;
+    require Net::WebSocket::Message;
+
+    my $frame_class = Net::WebSocket::FrameTypeName::get_module($frame_type);
+    Module::Load::load($frame_class) if !$frame_class->can('new');
+
+    return Net::WebSocket::Message->new(
+        $frame_class->new(
+            payload => $payload,
+            $self->FRAME_MASK_ARGS(),
+        ),
+    );
+}
+
 sub check_heartbeat {
     my ($self) = @_;
 
@@ -238,7 +257,7 @@ sub _got_non_continuation_during_fragment {
 
     $self->_write_frame($err_frame);
 
-    die Net::WebSocket::X->create( 'ReceivedBadControlFrame', $msg );
+    die Net::WebSocket::X->create( 'ReceivedBadDataFrame', $msg );
 }
 
 sub _verify_not_closed {
@@ -256,7 +275,15 @@ sub _handle_control_frame {
 
     if ($type eq 'close') {
         if (!$self->{'_sent_close_frame'}) {
-            $self->_close_with_frame($frame);
+            my ($code, $reason) = $frame->get_code_and_reason();
+
+            $self->_close_with_frame(
+                Net::WebSocket::Frame::close->new(
+                    code => $code,
+                    reason => $reason,
+                    $self->FRAME_MASK_ARGS(),
+                ),
+            );
         }
 
         if ($self->{'_received_close_frame'}) {
